/* ---------------------------------------------------------------------------------------------------------------------------- */
/* CREATE THE DATABASE BEFORE EXECUTING THIS SCRIPT */
/* CREATE DB ROLES OpIRoleDBManager, OpIRoleDBClient
	FOR SPECIFIED DATABASE*/
/* This procedure adds or replaces Roles, granting each selected permissions from Fixed Roles 
	so that DB Users made Members will inherit those permissions */
/* USE A SEPARATE DB ROLE SETUP SQL FILE FOR EACH OpI DATABASE */
/* ---------------------------------------------------------------------------------------------------------------------------- */

/* Security Architecture:  
	https://learn.microsoft.com/en-us/sql/relational-databases/security/authentication-access/getting-started-with-database-engine-permissions?view=sql-server-ver16
	https://www.mssqltips.com/sqlservertip/2699/sql-server-user-defined-server-roles/ */



/* SET UP Role OpIRoleDBManager for specified Database */
USE SQLServerToolDB; 	-- Specify the Database to which these DB Roles apply
GO
DECLARE	@roleName NVARCHAR(max) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE	@databaseName NVARCHAR(max) = NULL
DECLARE	@query NVARCHAR(max) = NULL
DECLARE	@schemaName NVARCHAR(max) = NULL
SET		@roleName=N'OpIRoleDBManager'
SET		@activeNow = N'Yes'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No
SET		@databaseName=N'SQLServerToolDB'
SET		@schemaName=N'dbo'

IF		@activeNow = N'Yes'
BEGIN
	IF	@replaceNow = N'Yes'
	BEGIN
		/* Drop previously-set Database Fixed Role membership */
		SET @query = N'ALTER ROLE db_owner			'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_securityadmin	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_accessadmin	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_backupoperator	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_ddladmin 		'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_datawriter 	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_datareader 	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_denydatawriter '	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_denydatareader '	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		/* User Roles OpIDeveloper and OpIUser may or may not exist; Remove this MEMBER if the Role exists. */
		SET @query=N'IF NOT DATABASE_PRINCIPAL_ID(''OpIDeveloper'') IS NULL '+
			N'ALTER ROLE OpIDeveloper 		'			+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query=N'IF NOT DATABASE_PRINCIPAL_ID(''OpIUser'') IS NULL '+
			N'ALTER ROLE OpIUser 		'			+ N'DROP MEMBER '			+ @roleName + N';';		EXEC sp_executesql @query;
	END

	SET @query=N'IF DATABASE_PRINCIPAL_ID(''' + @roleName + N''') IS NULL ' +
		N'CREATE ROLE [' + @roleName + '] AUTHORIZATION db_owner;'; EXEC sp_executesql @query;
	/* Configure database-level permissions to the role using GRANT, DENY, REVOKE: https://learn.microsoft.com/en-us/sql/t-sql/statements/create-role-transact-sql?view=sql-server-ver16 */

	/* Database Fixed Role db_securityadmin */
	SET @query = N'GRANT ALTER ANY APPLICATION ROLE '				+ N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	SET @query = N'GRANT ALTER ANY ROLE '							+ N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	SET @query = N'GRANT CREATE ROLE '								+ N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	SET @query = N'GRANT CREATE SCHEMA '							+ N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Database Fixed Role db_accessadmin */
	SET @query = N'GRANT ALTER ANY USER '							+ N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	SET @query = N'GRANT CONNECT '									+ N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Database Fixed Role db_ddladmin */
	SET @query = N'GRANT CREATE FUNCTION '							+ N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	SET @query = N'GRANT CREATE PROCEDURE '							+ N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	SET @query = N'GRANT CREATE TABLE '								+ N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	SET @query = N'GRANT CREATE VIEW '								+ N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Database Fixed Role db_datawriter */
	SET @query = N'GRANT DELETE ON DATABASE::'		+ @databaseName + N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	SET @query = N'GRANT INSERT ON DATABASE::'		+ @databaseName + N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	SET @query = N'GRANT UPDATE ON DATABASE::'		+ @databaseName + N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Database Fixed Role db_datareader */
	SET @query = N'GRANT SELECT ON DATABASE::'		+ @databaseName + N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* User Role OpIDeveloper */
	SET @query = N'GRANT CONTROL ON SCHEMA::'		+ @schemaName	+ N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
END
GO


/* SET UP Role OpIRoleDBClient for specified Database */
USE SQLServerToolDB; 	-- Specify the Database to which these DB Roles apply
GO
DECLARE	@roleName NVARCHAR(max) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE	@databaseName NVARCHAR(max) = NULL
DECLARE	@query NVARCHAR(max) = NULL
DECLARE	@schemaName NVARCHAR(max) = NULL
SET		@roleName=N'OpIRoleDBClient'
SET		@activeNow = N'Yes'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No
SET		@databaseName=N'SQLServerToolDB'
SET		@schemaName=N'dbo'

IF		@activeNow = N'Yes'
BEGIN
	IF	@replaceNow = N'Yes'
	BEGIN
		/* Drop previously-set Database Fixed Role membership */
		SET @query = N'ALTER ROLE db_owner			'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_securityadmin	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_accessadmin	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_backupoperator	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_ddladmin 		'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_datawriter 	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_datareader 	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_denydatawriter '	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_denydatareader '	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		/* User Roles OpIDeveloper and OpIUser may or may not exist; Remove this MEMBER if the Role exists. */
		SET @query=N'IF NOT DATABASE_PRINCIPAL_ID(''OpIDeveloper'') IS NULL '+
			N'ALTER ROLE OpIDeveloper 		'			+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query=N'IF NOT DATABASE_PRINCIPAL_ID(''OpIUser'') IS NULL '+
			N'ALTER ROLE OpIUser 		'			+ N'DROP MEMBER '			+ @roleName + N';';		EXEC sp_executesql @query;
	END

	SET @query=N'IF DATABASE_PRINCIPAL_ID(''' + @roleName + N''') IS NULL ' +
		N'CREATE ROLE [' + @roleName + '] AUTHORIZATION db_owner;'; EXEC sp_executesql @query;
	/* Configure database-level permissions to the role using GRANT, DENY, REVOKE: https://learn.microsoft.com/en-us/sql/t-sql/statements/create-role-transact-sql?view=sql-server-ver16 */

	/* Database Fixed Role db_accessadmin */
	SET @query = N'GRANT CONNECT '									+ N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Database Fixed Role db_datareader */
	SET @query = N'GRANT SELECT ON DATABASE::'		+ @databaseName + N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Database Fixed Role db_denydatawriter */
	SET @query = N'DENY DELETE ON DATABASE::'		+ @databaseName + N' TO [' + @roleName + N'] CASCADE;';				EXEC sp_executesql @query;
	SET @query = N'DENY INSERT ON DATABASE::'		+ @databaseName + N' TO [' + @roleName + N'] CASCADE;';				EXEC sp_executesql @query;
	SET @query = N'DENY UPDATE ON DATABASE::'		+ @databaseName + N' TO [' + @roleName + N'] CASCADE;';				EXEC sp_executesql @query;
	/* User Role OpIUser */
	SET @query = N'GRANT EXECUTE ON SCHEMA::'		+ @schemaName	+ N' TO [' + @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	END
GO
